#!/usr/bin/python3

from queue import Queue
from threading import Thread
import socketserver
import subprocess


num_gpu_threads = 4
HOST, PORT = "192.168.42.4", 1234

q = Queue()

comm_module = None

def log(text):
    import datetime
    print (str(datetime.datetime.now()) + ': ' + text)

def run_command(gpu_index, data):
    command = comm_module.get_command(gpu_index, data)
    
    if not command:
        return

    log ("Spawning '{}' for '{}' on GPU {}".format(command[0], data, gpu_index))
    try:
        subprocess.check_output(command, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        log (e.cmd[0] + " failed: " + str(e.stdout))
    else:
        log ("Terminated '{}' successfully on '{}'".format(command[0], data))

def gpu_worker(index):
    from time import sleep
    while True:
        item = q.get()
        log ("Taking " + item)
        try:
            run_command(index, item)
        except Exception as e:
            log ("Failed to run: " + str(e))
        q.task_done()
        log ("{} tasks remaining".format(q.qsize()))


class GPUTCPHandler(socketserver.StreamRequestHandler):
    def handle(self):
        for line in self.rfile.readlines():
            data = line.strip().decode('utf8')
            if data:
                log ("Queuing " + data)
                q.put(data)


if __name__ == "__main__":
    import socketserver
    from importlib import import_module
    import sys

    sys.dont_write_bytecode = True
    
    if len(sys.argv) > 1:
        module = sys.argv[1]
    else:
        module = 'comm_sleep'
    
    try:
        comm_module = import_module(module)
    except ImportError as e:
        log ("Cannot load command: %s" % (str(e)))
        sys.exit(1)
    
    log ("Starting with command from '{}'".format(module))

    # spawn the consumer threads
    for i in range(num_gpu_threads):
        t = Thread(target=gpu_worker, args=(i,))
        t.daemon = True
        t.start()
    
    # start the listening server
    server = socketserver.TCPServer((HOST, PORT), GPUTCPHandler)
    server.serve_forever()
