import os
import web
import base64
import re
from web import form
from operator import itemgetter

render = web.template.render('templates/', globals={"basestring": basestring, "os": os})

urls = (
  '/', 'index',
)

basedir = '/mnt/titan-samba/zone/titan/autoprocess/motioncor2/'
app       = web.application(urls, globals())

os.chdir(os.path.dirname(__file__))

def listalldirs():
    return listdirs(basedir)

def listdirs(curdir):
    # format: [{"path": "/a", "data": [{"path": "/a/b", "data": "b"}]}]
    return [{"path": os.path.join(curdir, name), "data": listdirs(os.path.join(curdir, name))} for name in os.listdir(curdir) if os.path.isdir(os.path.join(curdir, name))]

myform = form.Form(
    form.Textbox("Directory to configure",
        form.notnull,
        form.Validator('Must be an existing directory', lambda d:os.path.isdir(d)),
        size=80,
        id='indir'),
    form.Textbox('Gain filename (*.dm4 if emtpy)',
        id='gain'),
    form.Textbox('Binning',
        form.notnull,
        form.regexp('\d+', 'Must be an integer'),
        id='binning'),
    form.Textbox('Dose per frame',
        form.notnull,
        form.regexp('(\d.)+', 'Must be a real'),
        id='dose'),
    form.Textbox('Pixel size',
        form.notnull,
        form.regexp('(\d.)+', 'Must be a real'),
        id='pixelsize'),
    form.Textbox('Initial frames to ignore',
        form.notnull,
        form.regexp('\d+', 'Must be an integer'),
        id='ignore'),
    form.Textbox('Group frames before aligning',
        form.notnull,
        form.regexp('\d+', 'Must be an integer'),
        value = 1,
        id='group'),
    form.Checkbox('Start processing of all TIFFs without MRC',
        checked=True,
        value="bug-dummy1",
        id='start'))

class index:

    def GET(self):
        form = myform()
        return render.motioncor2(form, listalldirs())

    def POST(self):
        form = myform()
        if not form.validates():
            return render.motioncor2(form, listalldirs())
        else:
            import os
            
            confdir   = form['Directory to configure'].value
            gain      = form['Gain filename (*.dm4 if emtpy)'].value
            binning   = form['Binning'].value
            dose      = form['Dose per frame'].value
            pixelsize = form['Pixel size'].value
            ignore    = form['Initial frames to ignore'].value
            group     = form['Group frames before aligning'].value
            start     = form['Start processing of all TIFFs without MRC'].checked
            
            if not gain:
                import glob
                names = glob.glob(os.path.join(confdir, '*.dm4'))
                if names:
                    gain = names[0][len(confdir)+1:]
                else:
                    return "Could not match *.dm4 for gain file."
            
            if not os.access(os.path.join(confdir, gain), os.F_OK):
                return "Could not find gain file."
            
            self.write_yaml(confdir, gain[:-3] + "mrc", binning, dose, pixelsize, ignore, group)
            
            os.spawnl (os.P_NOWAIT, './dm4tomrc.sh', './dm4tomrc.sh', basedir, confdir, gain)
            
            if start:
                os.spawnl (os.P_NOWAIT, './launch-motioncor2.sh', './launch-motioncor2.sh', basedir, confdir)
            
            return "Parameters OK, gain file converting to MRC."

    def write_yaml(self, basedir, gain, binning, dose, pixelsize, ignore, group):
        import yaml
        
        conffile = basedir + '/motioncor2-params.yaml'
        data = {
            'gain': gain.encode('ASCII'),
            'binning': binning.encode('ASCII'),
            'dose': dose.encode('ASCII'),
            'pixelsize': pixelsize.encode('ASCII'),
            'ignore': ignore.encode('ASCII'),
            'group': group.encode('ASCII'),
        }

        stream = open(conffile, 'w')
        yaml.dump(data, stream, default_flow_style=False)

if __name__ == "__main__":
    app.run()
